#ifndef SLMATH_RANDOM_H
#define SLMATH_RANDOM_H

#include <slmath/float_util.h>
#include <slmath/mtrnd.h>

SLMATH_BEGIN()

/**
 * \defgroup random_util Random number and distribution generation.
 * @ingroup slmath
 */
/*@{*/

/**
 * Random number generator based on Mersenne Twister.
 *
 * Note naming convention: This class is starting with capital R since 
 * it is initialized properly in the constructor, unlike vec4, etc., classes.
 */
class Random
{
public:
	/**
	 * Initializes random number generator with current time.
	 */
	Random();

	/**
	 * Initializes random number generator with specified seed value.
	 */
	Random( int seed );

	/** 
	 * Returns random floating point number in range [0,1].
	 */
	float	randomFloat();

	/** 
	 * Returns random floating point number in half-open range [0,1).
	 */
	float	randomFloatOpen();

	/**
	 * Returns random integer.
	 */
	int		randomInt();

	/**
	 * Returns the seed value which was used to initialize this random number generator.
	 */
	int		seed() const;

private:
	MersenneTwist	m_mt;
	int				m_seed;
};

/*@}*/

SLMATH_END()

#endif

// This file is part of 'slmath' C++ library. Copyright (C) 2009 Jani Kajala (kajala@gmail.com). See http://sourceforge.net/projects/slmath/
