#ifndef SLMATH_RAND_UTIL_H
#define SLMATH_RAND_UTIL_H

#include <slmath/vec3.h>
#include <slmath/Random.h>

SLMATH_BEGIN()

/** 
 * Returns random floating point number in range [0,1].
 * @param seed Random number generator
 * @ingroup random_util
 */
float	randomFloat( Random& seed );

/** 
 * Returns signed random floating point number in range [-1,1].
 * @param seed Random number generator
 * @ingroup random_util
 */
float	randomFloatSigned( Random& seed );

/** 
 * Returns uniformly distributed random point on unit sphere. 
 * @param seed Random number generator
 * @ingroup random_util
 */
vec3	randomPointOnSphere( Random& seed );

/** 
 * Returns uniformly distributed random point inside unit sphere. 
 * @param seed Random number generator
 * @ingroup random_util
 */
vec3	randomPointInSphere( Random& seed );

/**
 * Produces n stratified samples on hemisphere surface.
 * @param normal Hemisphere direction. Must be unit vector.
 * @param angle Cone angle to limit hemisphere (pass pi/2 to sample on full hemisphere surface).
 * @param n Number of samples to generate.
 * @param out [out] Receives generated samples.
 * @param seed [inout] Random number generator.
 * @ingroup random_util
 */
void	randomSamplePointsOnHemisphere( const vec3& normal, float angle, size_t n, vec3* out, Random& seed );

/**
 * Produces n stratified samples within hemisphere volume.
 * @param normal Hemisphere direction. Must be unit vector.
 * @param angle Cone angle to limit hemisphere (pass pi/2 to sample within full hemisphere volume).
 * @param n Number of samples to generate.
 * @param out [out] Receives generated samples.
 * @param seed [inout] Random number generator.
 * @ingroup random_util
 */
void	randomSamplePointsInHemisphere( const vec3& normal, float angle, size_t n, vec3* out, Random& seed );

SLMATH_END()

#endif 

// This file is part of 'slmath' C++ library. Copyright (C) 2009 Jani Kajala (kajala@gmail.com). See http://sourceforge.net/projects/slmath/
