#ifndef SLMATH_PP_H
#define SLMATH_PP_H

#include <math.h>
#include <float.h>
#include <assert.h>
#include <slmath/slmath_configure.h>

// Verify requested configuration for this build:
// SSE2 extensions supported for Visual Studio 2005 and newer
// (we *should* check CPU type at runtime also but we ignore it for now)
#if (_MSC_VER >= 1500)
	#if defined(SLMATH_SIMD) && (defined(_M_X64) || _M_IX86_FP >= 2)
		#define SLMATH_SSE2_MSVC
	#endif
#endif

// SIMD macros
#ifdef SLMATH_SSE2_MSVC
	#include <xmmintrin.h>

	#define SLMATH_M128 __m128
	#define SLMATH_ALIGN16 __declspec(align(16))
	#define SLMATH_MUL_PS(A,B) _mm_mul_ps(A,B)
	#define SLMATH_ADD_PS(A,B) _mm_add_ps(A,B)
	#define SLMATH_SUB_PS(A,B) _mm_sub_ps(A,B)
	#define SLMATH_LOAD_PS1(A) _mm_load_ps1(A)
#else 
	// SIMD emulation with standard C++, so you can still use SIMD-macros even without SIMD support if you want
	#undef SLMATH_SIMD

	struct slmath_m128
	{
		float m[4]; 
		
		slmath_m128() {} 
		slmath_m128(float x) {m[0]=m[1]=m[2]=m[3]=x;} 
		slmath_m128(float x,float y,float z,float w) {m[0]=x;m[1]=y;m[2]=z;m[3]=w;} 
	};

	#define SLMATH_M128 slmath_m128
	#define SLMATH_ALIGN16
	#define SLMATH_MUL_PS(A,B) slmath_m128( (A).m[0]*(B).m[0], (A).m[1]*(B).m[1], (A).m[2]*(B).m[2], (A).m[3]*(B).m[3] )
	#define SLMATH_ADD_PS(A,B) slmath_m128( (A).m[0]+(B).m[0], (A).m[1]+(B).m[1], (A).m[2]+(B).m[2], (A).m[3]+(B).m[3] )
	#define SLMATH_SUB_PS(A,B) slmath_m128( (A).m[0]-(B).m[0], (A).m[1]-(B).m[1], (A).m[2]-(B).m[2], (A).m[3]-(B).m[3] )
	#define SLMATH_LOAD_PS1(A) slmath_m128(*(A))
#endif

// Use this instead of namespaces directly to enable easier porting for platforms without namespace support
#ifdef SLMATH_NAMESPACE
	#define SLMATH_BEGIN() namespace slmath {
	#define SLMATH_END() }
	#define SLMATH_USING() using namespace slmath;
#else
	#define SLMATH_BEGIN()
	#define SLMATH_END()
	#define SLMATH_USING()
#endif

#endif

// This file is part of 'slmath' C++ library. Copyright (C) 2009 Jani Kajala (kajala@gmail.com). See http://sourceforge.net/projects/slmath/
