#include <slmath/testframework.h>
#include <stdio.h>
#include <string.h>
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#if defined(_DEBUG) && defined(_MSC_VER) 
#include <crtdbg.h>
#endif

SLMATH_BEGIN()

/** 
 * Maximum length of the test ID string. 
 * Note that the constant is internal to the implementation (this file),
 * so it does not harm even if we hardcode the constant like this.
 */
static const int TESTID_MAX_LEN = 512;

void formatTestId( char* testid, const char* file, int line, const char* timestamp, const char* expr )
{
	sprintf_s( testid, TESTID_MAX_LEN, "TEST(%s): %s(%d) version %s", expr, file, line, timestamp );
}

void debugBreak()
{
#if defined(_DEBUG) && defined(_MSC_VER) 
	_CrtDbgBreak();
#endif
}

static void defaultOutput( void* /*context*/, const char* sz )
{
	printf( "%s", sz );
#ifdef WIN32
	OutputDebugStringA( sz );
#endif
}

bool executeTests( const char* libname, TestFunc_t testfunc, 
	int flags, TestOutputFunc_t output, void* context )
{
	if ( 0 == output )
		output = defaultOutput;

	const bool verbose = 0 != (TEST_VERBOSE & flags);
	char testid[TESTID_MAX_LEN] = "";

	try
	{
		if ( verbose )
		{
			output( context, "Testing " );
			output( context, libname );
			output( context, " ... " );
		}

		bool ok = testfunc( testid );
		if ( ok )
		{
			if ( verbose ) 
				output( context, "OK!\n" );
		}
		else
		{
			output( context, "FAILED " );
			output( context, testid );
		}

		return ok;
	}
	catch ( ... )
	{
		output( context, "EXCEPTION IN " );
		output( context, libname );
		output( context, " " );
		output( context, testid );
		return false;
	}
}

SLMATH_END()

// This file is part of 'slmath' C++ library. Copyright (C) 2009 Jani Kajala (kajala@gmail.com). See http://sourceforge.net/projects/slmath/
