#include <slmath/vec3.h>

SLMATH_BEGIN()

bool check( const vec3& v )
{
	return 
		v.x >= -FLT_MAX && v.x <= FLT_MAX &&
		v.y >= -FLT_MAX && v.y <= FLT_MAX &&
		v.z >= -FLT_MAX && v.z <= FLT_MAX;
}

vec3 normalize( const vec3& v )
{
	assert( check(v) );

	float len = length(v);
	assert( len >= FLT_MIN );
	
	float invlen = 1.f / len;
	vec3 res( v.x*invlen, v.y*invlen, v.z*invlen );
	assert( check(res) );
	return res;
}

vec3 cross( const vec3& a, const vec3& b )		
{
	assert( check(a) );
	assert( check(b) );

	// i    j     k
	// a.x  a.y   a.z
	// b.x  b.y   b.z
	vec3 res( a.y*b.z-a.z*b.y, a.z*b.x-a.x*b.z, a.x*b.y-a.y*b.x );
	assert( check(res) );
	return res;
}

vec3 facenormal_ccw( const vec3& v0, const vec3& v1, const vec3& v2 )
{
	vec3 a( v1.x-v0.x, v1.y-v0.y, v1.z-v0.z );
	vec3 b( v2.x-v0.x, v2.y-v0.y, v2.z-v0.z );
	vec3 res( a.y*b.z-a.z*b.y, a.z*b.x-a.x*b.z, a.x*b.y-a.y*b.x );
	const float len = length(res);
	assert( len >= FLT_MIN );
	res *= 1.f/len;
	return res;
}

vec3 facenormal_cw( const vec3& v0, const vec3& v1, const vec3& v2 )
{
	vec3 a( v2.x-v0.x, v2.y-v0.y, v2.z-v0.z );
	vec3 b( v1.x-v0.x, v1.y-v0.y, v1.z-v0.z );
	vec3 res( a.y*b.z-a.z*b.y, a.z*b.x-a.x*b.z, a.x*b.y-a.y*b.x );
	const float len = length(res);
	assert( len >= FLT_MIN );
	res *= 1.f/len;
	return res;
}

SLMATH_END()

// This file is part of 'slmath' C++ library. Copyright (C) 2009 Jani Kajala (kajala@gmail.com). See http://sourceforge.net/projects/slmath/
